/*
 * Billboard CGI main routines
 * Copyright (C) 1999 Open World Ltd
 * Written by Shevek <shevek@anarres.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <errno.h>
#include "library.h"
#include "rope.h"
#include "vector.h"
#include "utils.h"
#include "hash.h"
#include "cgi.h"
#include "log.h"

vector_t *config_vector(rope_t *path);

int
file_cat(char *hostname, char *ccase, char *file)
{
	FILE	*input;
	char	 line[BUFSIZ];
	rope_t	*r;

	r = rope_new(DIR_DATA "/");
	rope_add(r, hostname);
	rope_add(r, "/billboard/");
	rope_add(r, ccase);
	rope_add(r, "/fragments/");
	rope_add(r, file);
	
	input = fopen(rope_chars(r), "r");
	if (!input) {
		log("cgiconf", PRI_CRIT, "Could not open billboard file '%s': "
						"error was '%s'",
						rope_chars(r), strerror(errno));
		return FALSE;
	}

	printf("Content-type: text/html\r\n\r\n");
	while (fgets(line, BUFSIZ, input)) {
		printf("%s", line);
	}

	return TRUE;
}

void
file_pick(char *hostname, char *ccase, vector_t *v)
{
	char	*file;
	int		 len;
	int		 rand;
	int		 i;

	len = vector_length(v);
	if (len == 0) {
		log("cgiconf", PRI_CRIT, "Got no files in vector");
		exit(0);
	}
	srandom(getpid() + time(NULL));
	rand = random() >> 4;	/* Not much entropy in last bits */

	for (i = 0; i < len; i++) {
		file = vector_data(v, (rand + i) % len);
		if (file_cat(hostname, ccase, file)) return;
		log("cgiconf", PRI_ERR, "Failed to read file '%s'", file);
	}

	if (file_cat(hostname, ccase, "default.txt")) return;

	/* Not fatal, we're about to drop out anyhow */
	log("cgiconf", PRI_CRIT, "Could not read any files");
	printf("Content-type: text/html\n\n");
	printf("No billboard\n");
}

vector_t *
vector_get(char *hostname, char *ccase)
{
	rope_t		*r;
	vector_t	*v;

	r = rope_new(DIR_DATA "/");
	rope_add(r, hostname);
	rope_add(r, "/billboard/");
	rope_add(r, ccase);
	rope_add(r, "/config.txt");
	v = config_vector(r);
	rope_delete(r);
	// log("cgisys", PRI_DEBUG, "Got files %s", implode(v, ", "));
	// DPRINTF((stderr, "Got files %s\n", implode(v, ", ")));
	return v;
}

int
main(void)
{
	vector_t	*v;
	hash_t		*cgihash;
	char		*ccase;
	char		*hostname;

	library_init("billboard");
	
	ccase = getenv("CASE");
	if (!ccase) {
		cgihash = cgi_hash_new();
		ccase = hash_get(cgihash, "ccase");
	}
	if (!ccase) {
		log("cgiconf", PRI_CRIT, "No CASE in environment");
		ccase = "default";
	}
	ccase[strcspn(ccase, "/")] = 0;

	hostname = getenv(ENV_HOSTNAME);
	if (!hostname) {
		log("cgiconf", PRI_CRIT, "No " ENV_HOSTNAME " in environment");
		exit(0);
	}

	v = vector_get(hostname, ccase);

	file_pick(hostname, ccase, v);

	return 0;
}
