%{
/*
 * Booking dates form generator (simple version) - template parser
 * Copyright (C) 1999 Open World Ltd
 * Written by Shevek <shevek@anarres.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <time.h>
#include <stdio.h>
#include <unistd.h>
#include "library.h"
#include "rope.h"
#include "cgi.h"

void template_days(struct tm *tm);
void template_months(struct tm *tm);

struct tm	*today;
struct tm	*tomorrow;

%}

%option noyywrap
%option nounput

tARR_DAY_ST		\{ArrivalDayNumbers\}
tARR_DAY_END	\{\/ArrivalDayNumbers\}
tARR_MON_ST		\{ArrivalMonthYears\}
tARR_MON_END	\{\/ArrivalMonthYears\}

tDEP_DAY_ST		\{DepartureDayNumbers\}
tDEP_DAY_END	\{\/DepartureDayNumbers\}
tDEP_MON_ST		\{DepartureMonthYears\}
tDEP_MON_END	\{\/DepartureMonthYears\}

tARR_DAY		\{ArrivalDayNumber\}
tARR_MON		\{ArrivalMonthYear\}

tDEP_DAY		\{DepartureDayNumber\}
tDEP_MON		\{DepartureMonthYear\}

%%

{tARR_DAY_ST}		{ }
{tARR_DAY_END}		{ }
{tARR_MON_ST}		{ }
{tARR_MON_END}		{ }

{tDEP_DAY_ST}		{ }
{tDEP_DAY_END}		{ }
{tDEP_MON_ST}		{ }
{tDEP_MON_END}		{ }

{tARR_DAY}			{ template_days(today);			}
{tARR_MON}			{ template_months(today);		}

{tDEP_DAY}			{ template_days(tomorrow);		}
{tDEP_MON}			{ template_months(tomorrow);	}

%%

const char *months[] = {
	"January",
	"February",
	"March",
	"April",
	"May",
	"June",
	"July",
	"August",
	"September",
	"October",
	"November",
	"December"
};

void
template_times(void)
{
	time_t	 timebuf;

	time(&timebuf);
	today = gmtime(&timebuf);

	timebuf += 86400;
	tomorrow = gmtime(&timebuf);
}

void
template_stream(rope_t *pathstr)
{
	char	*path;

	path = rope_chars(pathstr);

	if (access(path, R_OK) != 0) {
		perror(path);
		cgi_fatal("Could not open template file");
	}
	yyin = fopen(path, "r");
	if (!yyin) {
		perror(path);
		cgi_fatal("Could not open template file");
	}
}

void
template_parse(rope_t *pathstr)
{
	template_times();
	template_stream(pathstr);

	yylex();
}

/* === */

void
template_days(struct tm *tm)
{
	int	 i;
	for (i = 0; i < 32; i++) {
		printf("<option");
		if (i == tm->tm_mday)
			printf(" selected");

		printf("> %.2d\n", i);
	}
}

void
template_months(struct tm *tm)
{
	int		 i;
	int		 mon;
	int		 off;

	for (i = tm->tm_mon; i < tm->tm_mon + 12; i++) {
		off = (int)(i / 12);	/* Integer rounding (leq divmod) */
		mon = i % 12;

		printf("<option");
		if (mon == tm->tm_mon)
			printf(" selected");

		printf(" VALUE=\"%s\"> ", months[mon]);
		printf("%s %d\n", months[mon], tm->tm_year + off + 1900);
	}
}
