/*
 * CGI countdown timer - main routines
 * Copyright (C) 1999 Open World Ltd
 * Written by Shevek <shevek@anarres.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include "library.h"
#include "hash.h"
#include "rope.h"
#include "cgi.h"
#include "smalloc.h"
#include "log.h"

/* Fingers out from here on */
#define STREQ(x, y) (strcasecmp(x, y) == 0)

typedef
struct _cd_info_t {
	char	*timezone;
	char	*digitset;
	int		 digit;
	time_t	 target;
} cd_info_t;

void
countdown_show(cd_info_t *info)
{
	time_t		 now;
	time_t		 interval;

	int			 days;
	int			 hours;
	int			 minutes;

	int			 val;
	char		*server;

	server = getenv(ENV_HOSTNAME);
	if (!server) {
		log("cgiconf", PRI_CRIT, "No \"" ENV_HOSTNAME
						"\" in environment");
		exit(0);
	}

	time(&now);
	interval = info->target - now;
	if (interval < 0) interval = 0;

	interval /= 60;						/* Work in minutes */
	days = interval / (24 * 60);		/* Integer arithmetic */
	interval %= (24 * 60);				/* Get remaining minutes */
	hours = interval / 60;				/* Integers again */
	minutes = interval % 60;			/* And the rest */

	switch(info->digit) {
		case 0: val = minutes % 10;		break;	/* Minute units */
		case 1: val = minutes / 10;		break;	/* Minute tens */
		case 2: val = hours % 10;		break;	/* Hour units */
		case 3: val = hours / 10;		break;	/* Hour tens */
		case 4: val = days % 10;		break;	/* Day units */
		case 5: val = (days/10)    %10;	break;	/* Day tens */
		case 6: val = (days/100)   %10;	break;	/* Day hundreds */
		case 7: val = (days/1000)  %10;	break;	/* Day thousands */
		case 8: val = (days/10000) %10;	break;	/* Day lots */
		case 9: val = (days/100000)%10;	break;	/* Day bignums */
		default:	val = 0;	/* Defeat compile warnings */
	}

	printf("Location: http://%s" DIR_RESOURCE
					"/countdown/%s/digits/%c.gif" "\n\n",
					server, info->digitset, ((char)(val + '0')) );
}

/* QUERY_STRING format: Traditional CGI a=b&c=d
 * Parameters:
 * case				- A funny, meaningless parameter
 * digitPosition	- digit to show, may be (hours|minutes)(tens|units)
 * All other parameters are ignored
 */
void
countdown_info_long(cd_info_t *info)
{
	hash_t	*cgihash;
	char	*temp;

	cgihash = cgi_hash_new();

/* Get the digit */
	temp = hash_get(cgihash, "digitPosition");
	if (!temp) {
		log("cgiconf", PRI_CRIT, "No 'digitPosition' in QUERY_STRING");
		exit(0);
	}
	if (strlen(temp) == 1)
		info->digit = (((unsigned char)(*temp)) + 700 - '0') % 10;
	else if (STREQ(temp, "minutesUnits"))
		info->digit = 0;
	else if (STREQ(temp, "minutesTens"))
		info->digit = 1;
	else if (STREQ(temp, "hoursUnits"))
		info->digit = 2;
	else if (STREQ(temp, "hoursTens"))
		info->digit = 3;
	else if (STREQ(temp, "daysUnits"))
		info->digit = 4;
	else if (STREQ(temp, "daysTens"))
		info->digit = 5;
	else if (STREQ(temp, "daysHundreds"))
		info->digit = 6;
	else if (STREQ(temp, "daysThousands"))
		info->digit = 7;
	else if (STREQ(temp, "daysLots"))
		info->digit = 8;
	else if (STREQ(temp, "daysBignums"))
		info->digit = 9;
	else {
		log("cgiconf", PRI_CRIT, "digitPosition parameter `%s' is "
						"invalid", temp);
		exit(0);
	}

	/* XXX This is OW-incompatible, but that isn't critical */
	info->digitset = hash_get(cgihash, "case");
	if (!info->digitset)
		info->digitset = "default";

	temp = hash_get(cgihash, "target");
	if (temp)
		info->target = atol(temp);
	else {
		log("cgiconf", PRI_CRIT, "No 'target' in QUERY_STRING");
		exit(0);
	}
}

int
main(void)
{
	cd_info_t	 ci;

	library_init("countdown");
	countdown_info_long(&ci);
	countdown_show(&ci);

	return 0;
}
