/*
 * Web site error daemon - runtime execution handlers
 * Copyright (C) 1999 Open World Ltd
 * Written by Shevek <shevek@anarres.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sys/types.h>
#include <regex.h>
#include "library.h"
#include "vector.h"
#include "smalloc.h"
#include "hash.h"
#include "wregex.h"
#include "servicehash.h"
#include "regexlist.h"
#include "runtime.h"
#include "dsyslog.h"

typedef
struct _runtime_data_t {
	regexlist_t	*rlist;
	hash_t		*shash;
} runtime_data_t;

static runtime_data_t *
runtime_data_new(regexlist_t *rlist, hash_t *shash)
{
	runtime_data_t	*r;
	r = (runtime_data_t *)smalloc(sizeof(runtime_data_t));
	r->rlist = rlist;
	r->shash = shash;
	return r;
}

static void
runtime_data_delete(runtime_data_t *r)
{
	regexlist_delete(r->rlist);
	servicehash_delete(r->shash);
	sfree(r);
}

void
runtime_delete(vector_t *r)
{
	vector_delete(r);
}

vector_t *
runtime_new(char *path)
{
	vector_t	*r;
	r = vector_new((vector_delete_t)runtime_data_delete, 0);
	dsyslog_read(r, path);
	return r;
}

void
runtime_push_back(vector_t *r, regexlist_t *rlist, hash_t *shash)
{
	runtime_data_t	*d;
	d = runtime_data_new(rlist, shash);
	vector_push_back(r, d);
}

void
runtime_execute(vector_t *v, errmsg_t *msg)
{
	runtime_data_t	*r;
	int				 len;
	int				 i;

	len = vector_length(v);
	for (i = 0; i < len; i++) {
		r = vector_data(v, i);
		if (regexlist_execute(r->rlist, msg->site) == 0) {
			servicehash_execute(r->shash, msg);
		}
	}
}
