#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include <time.h>
#include <sys/time.h>
#include "vector.h"
#include "cgi.h"
#include "smalloc.h"
#include "utils.h"
#include "rope.h"
#include "log.h"

#ifdef DPRINTF
#undef DPRINTF
#endif
#define DPRINTF(x) do { } while(0)

#define datafile_data_vector(line) (explode((line), "|"))
#define datafile_match_strings(h, n) (strstr((h), (n)))

char	 timestr[24];

/* haystack, needle */
static int
datafile_match_vectors_str( vector_t	*datavec,
							vector_t	*inputvec,
							int mnf, int mxf)
{
	int	 i, j;
	// int	 val = 0;

	DPRINTF((stderr, "Checking string\n"));

	if (mxf == -1) mxf = vector_length(datavec) - 1;

	for (i = 0; i < vector_length(inputvec); i++) {
		for (j = mnf; j <= mxf; j++) {
			if (!vector_data(datavec, j)) {
				log("cgisys", PRI_ALERT, "No data field in vector at "
								"offset %d in '%s'",
								j, implode(datavec, "|"));
				continue;
			}
			if (datafile_match_strings(
					vector_data(datavec, j),
					vector_data(inputvec, i))) {
				DPRINTF((stderr, "Got a match at '%s'\n",
								(char *)vector_data(datavec, j)));
				return 1;
				// val++;
			}
		}
	}

	return 0;
	// return val;
}

static int
datafile_match_vectors_time(vector_t *datavec, int mnf, int mxf)
{
	char	*mnd;
	char	*mxd;

	mnd = vector_data(datavec, mnf);
	mxd = vector_data(datavec, mxf);

	DPRINTF((stderr, "Checking time\n"));
	/* This is getting hackish, but it really isn't the job of this
	 * app. At least this version works. */
	if (!mnd || !mxd) {
		log("cgisys", PRI_ALERT, "No time field in vector at "
						"offset %d or %d in '%s'",
						mnf, mxf, implode(datavec, "|"));
	}
	if (strcmp(timestr, mnd) < 0) return 0;
	if (strcmp(timestr, mxd) > 0) return 0;
	return 1;
}

static int
datafile_match_vectors(vector_t *datavec, vector_t *inputvec,
				int mnf, int mxf)
{
	if (inputvec) {
		if (vector_length(inputvec) == 0) return 1;
		return datafile_match_vectors_str(datavec, inputvec, mnf, mxf);
	}
	return datafile_match_vectors_time(datavec, mnf, mxf);
}

static inline void
datafile_do_time(void)
{
	time_t		 timebuf;
	struct tm	*tm;

	time(&timebuf);
	tm = gmtime(&timebuf);
	strftime(timestr, 22, "%Y%m%d%H%M", tm);
	DPRINTF((stderr, "Building time\n"));
}

static vector_t *
datafile_input_vector (char *str)
{
	vector_t	*inputvec;
	char		*cp;
	char		*sp;
	char		*base;

	if (strcmp(str, "RESERVED_NOW") == 0) {
		datafile_do_time();
		return NULL;
	}

	DPRINTF((stderr, "Making input vector from '%s'\n", str));
	inputvec = vector_new(sfree, 0);

	base = str;
	sp = str;
	for (cp = str; *cp; cp++) {
		if (isupper(*cp))
			*sp++ = tolower(*cp);
		else if (islower(*cp))
			*sp++ = *cp;
		else if (isspace(*cp)) {
			*sp++ = '\0';
			if (strlen(base)) {
				vector_push_back(inputvec, sstrdup(base));
				DPRINTF((stderr, "Got string '%s'\n", base));
			}
			base = sp;
		}
	}

	/* Grab last word */
	if (strlen(base)) {
		vector_push_back(inputvec, sstrdup(base));
		DPRINTF((stderr, "Got string '%s'\n", base));
	}

	DPRINTF((stderr, "Got %d strings\n", vector_length(inputvec)));

	return inputvec;
}

static FILE *
datafile_stream (rope_t *pathstr)
{
	FILE	*input;
	char	*file;			/* Path to datafile */

	file = rope_chars(pathstr);

	if (access(file, R_OK) != 0) {
		log("cgiconf", PRI_CRIT, "Could not open datafile '%s' for "
						"reading: %s", file, strerror(errno));
		exit(0);
	}

	input = fopen(file, "r");
	if (!input) {
		log("cgiconf", PRI_CRIT, "Could not open datafile '%s' for "
						"reading: %s", file, strerror(errno));
		exit(0);
	}

	return input;
}

vector_t *
datafile_process(rope_t *pathstr, char *search, int mnf, int mxf)
{
	char		 line[BUFSIZ];	/* Input line (fgets()) */
	char		 copy[BUFSIZ];	/* Input line (fgets()) */
	FILE		*input;			/* Input FILE* */
	int			 value;
	vector_t	*results;		/* Quadratic results vector */
	vector_t	*datavec;		/* Data vector */
	vector_t	*inputvec;		/* Input vector */

	inputvec = datafile_input_vector(search); /* Get words */
	results = vector_new((delete_t)vector_delete, 0);	/* Allocate */
	input = datafile_stream(pathstr);				/* Get stream */

/* Process the file */
	while (fgets(line, BUFSIZ, input)) {
		line[strcspn(line, "\r\n")] = '\0';
		DPRINTF((stderr, "Searching '%s'\n", line));

		strcpy(copy, line);
		lower_case(copy);
		datavec = datafile_data_vector(copy);

		value = datafile_match_vectors(datavec, inputvec, mnf, mxf);

		if (value)
			vector_push_back(results, explode(line, "|"));
		else
			vector_delete(datavec);
	}

	if (inputvec)
		vector_delete(inputvec);
	fclose(input);

	return results;
}

vector_t *
datafile_read(hash_t *cgihash, rope_t *pathstr)
{
	char	*search;
	int		 minfield;
	int		 maxfield;
	char	*temp;

/* Match fields */
	if ((temp = hash_get(cgihash, "Match"))) {
		if (*temp == '*') {
			minfield = 0;
			maxfield = -1;
		}
		else {
			minfield = atol(temp) - 1;
			maxfield = minfield;
		}
	}
	else if ((temp = hash_get(cgihash, "Range"))) {
		minfield = atol(temp);
		temp = strchr(temp, ' ') - 1;
		if (temp)
			maxfield = atol(++temp) - 1;
		else {
			log("cgiconf", PRI_CRIT, "Range expects two values");
			exit(0);
		}
	}
	else {
		log("cgiconf", PRI_CRIT, "No 'Match' or 'Range' parameter in "
						"cgi hash table");
		exit(0);
	}

	DPRINTF((stderr, "Minf %d, Maxf %d\n", minfield, maxfield));

/* Search text */
	search = hash_get(cgihash, "Text");
	if (!search) {
		log("cgiconf", PRI_CRIT, "No 'Text' parameter in "
						"cgi hash table");
		exit(0);
	}

	return datafile_process(pathstr, search, minfield, maxfield);
}
